;-------------------------------------------------------------------------
;
; Nikon ML-L1 / ML-L3 IR remote control replacement
;
; Author :  San Bergmans
; Date   :  May 2008
; Version:	1
; Target :  ATtiny13
;
;-------------------------------------------------------------------------


.INCLUDE <tn13def.inc>

;-------------------------------------------------------------------------
;  Declarations
;-------------------------------------------------------------------------

.EQU	BURST0=9					; Carrier low counter reload value
.EQU	BURST1=9					; Carrier high counter reload value
.EQU	CORRECTION=4				; First cycle overhead correction
.EQU	OUTMASK0=0b11111101			; Output mask 0 level
.EQU	OUTMASK1=0b11111111			; Output mask 1 level

.DEF	BDELAY=R16					; Burst delay counter
.DEF	BCOUNT=R17					; Burst counter
.DEF	LEDSTAT=R18					; LED status (11111110=off)
.DEF	ECOUNT=R19					; End delay counter

.CSEG

;-------------------------------------------------------------------------
;  Vector space
;-------------------------------------------------------------------------

.ORG 0
			RJMP	RESET			; Let's get started

			RETI					; External interrupt not used

			RETI					; Pin Change interrupt not used

;-------------------------------------------------------------------------
;  Let's get started
;-------------------------------------------------------------------------

RESET:
			LDI		R16,LOW(RAMEND)	; Init stack
			OUT		SPL,R16

			LDI		R16,0b10000000	; Set clock prescaler to /4
			OUT		CLKPR,R16
			LDI		R16,0b00000010
			OUT		CLKPR,R16

			LDI		R16,OUTMASK0	; PortB 0 = output, rest is input
			OUT		PORTB,R16
			COM		R16
			OUT		DDRB,R16

			CLR		R16				; Disable ADC
			OUT		ADMUX,R16
			OUT		ADCSRA,R16
			OUT		ADCSRB,R16		; Disable Comparator

;-------------------------------------------------------------------------
;  Processor spends most of its time asleep.
;  It only wakes up to transmit an IR command and falls back asleep
;  right after it.
;-------------------------------------------------------------------------

GOODNIGHT:
			LDI		R16,0b00100000	; Enable pin change interrupt
			OUT		GIMSK,R16
			LDI		R16,0b00000001
			OUT		PCMSK,R16
			SEI

			LDI		R16,0b00110000	; Enable power down mode
			OUT		MCUCR,R16			
			SLEEP					; Nighty, night

;-------------------------------------------------------------------------
;  Sleeping beauty has been kissed
;-------------------------------------------------------------------------

			CLI
			CLR		R16				; Disable sleep mode
			OUT		MCUCR,R16

			LDI		BDELAY,BURST0	; Delay about 6ms
			LDI		BCOUNT,250	
			LDI		LEDSTAT,OUTMASK0
			RCALL	SENDBURST

			RCALL	COMMAND			; Send command twice
			RCALL	COMMAND

LETGO:		IN		R16,PINB		; Is key still down?
			ANDI	R16,0b00000001
			BREQ	LETGO			; Not yet!
			
			RJMP	GOODNIGHT		; Time to get back to sleep

;-------------------------------------------------------------------------
;  Send BCOUNT pulses.
;  BCOUNT   : Holds number pulses to send
;  BDELAY   : Holds high/low delay times
;  LEDSTAT  : Holds mark (11111111) space (11111101) flag
;-------------------------------------------------------------------------

NEXTPULSE:	LDI		BDELAY,BURST0	; Reload delay counter for Low period
SENDBURST:	DEC		BDELAY			; Low period delay loop
			BRNE	SENDBURST
			NOP						; Fractional compensation
			OUT		PORTB,LEDSTAT	; Set or Clr output bit
			NOP						; OUT is one cycle, SBI was two
			LDI		BDELAY,BURST1	; Reload delay counter for High period
BURSTLOOP:	DEC 	BDELAY			; High period delay loop
			BRNE	BURSTLOOP
			CBI		PORTB,1			; Clr output bit
			DEC 	BCOUNT			; Decrement burst pulse counter
			BRNE	NEXTPULSE		; Do until all pulses are sent
			RET

;-------------------------------------------------------------------------
;  Send the simple command once
;-------------------------------------------------------------------------

COMMAND:	LDI		BDELAY,BURST0-CORRECTION ; Setup initial delay counter
			LDI		BCOUNT,76		; Start burst is 76 periods
			LDI		LEDSTAT,OUTMASK1 ; LED must be on
			RCALL	SENDBURST		; Send this burst

			LDI		BDELAY,BURST0-CORRECTION ; First mark is far too long
			LDI		BCOUNT,250		; to be sent at once, so I'll do this
			LDI		LEDSTAT,OUTMASK0 ; in 5 steps!
			RCALL	SENDBURST
			LDI		BDELAY,BURST0-CORRECTION ; Second 250us delay
			LDI		BCOUNT,250	
			LDI		LEDSTAT,OUTMASK0
			RCALL	SENDBURST
			LDI		BDELAY,BURST0-CORRECTION ; Third 250us delay
			LDI		BCOUNT,250	
			LDI		LEDSTAT,OUTMASK0
			RCALL	SENDBURST
			LDI		BDELAY,BURST0-CORRECTION ; Forth 250us delay
			LDI		BCOUNT,250	
			LDI		LEDSTAT,OUTMASK0
			RCALL	SENDBURST
			LDI		BDELAY,BURST0-CORRECTION ; Final delay of
			LDI		BCOUNT,64		; 64 periods, makes a total of 1064
			LDI		LEDSTAT,OUTMASK0
			RCALL	SENDBURST		; Which equals 28 ms

			LDI		BDELAY,BURST0-CORRECTION ; Now send a burst of
			LDI		BCOUNT,15		; 15 periods
			LDI		LEDSTAT,OUTMASK1
			RCALL	SENDBURST

			LDI		BDELAY,BURST0-CORRECTION ; Send a space of
			LDI		BCOUNT,60		; 60 periods
			LDI		LEDSTAT,OUTMASK0
			RCALL	SENDBURST

			LDI		BDELAY,BURST0-CORRECTION ; Send a burst of
			LDI		BCOUNT,15		; 15 periods again
			LDI		LEDSTAT,OUTMASK1
			RCALL	SENDBURST

			LDI		BDELAY,BURST0-CORRECTION ; Send a space of
			LDI		BCOUNT,136		; 136 periods
			LDI		LEDSTAT,OUTMASK0
			RCALL	SENDBURST

			LDI		BDELAY,BURST0-CORRECTION ; Send final burst of
			LDI		BCOUNT,15		; 15 periods
			LDI		LEDSTAT,OUTMASK1
			RCALL	SENDBURST

			LDI		LEDSTAT,OUTMASK0
			LDI		ECOUNT,10
ENDDELAY:	LDI		BDELAY,BURST0-CORRECTION ; Send a total delay of
			LDI		BCOUNT,240		; 10 x 240 periods (63.2 ms)
			RCALL	SENDBURST
			DEC		ECOUNT
			BRNE	ENDDELAY		; Do until end delay is over

			RET

